@@ -1,6 +1,5 @@
 var shinyTree = function(){
   callbackCounter = 0;
-  sttypes = null;
 
   var treeOutput = new Shiny.OutputBinding();
   $.extend(treeOutput, {
@@ -27,13 +26,37 @@ var shinyTree = function(){
       if ($elem.data('st-types') === 'TRUE'){
         plugins.push('types');
       }
-      
-      var tree = $(el).jstree({'core' : { 
-        "check_callback" : ($elem.data('st-dnd') === 'TRUE'), 
-        'themes': {'name': $elem.data('st-theme'), 'responsive': true, 'icons': ($elem.data('st-theme-icons') === 'TRUE'), 'dots': ($elem.data('st-theme-dots') === 'TRUE') }
-          },
-          "types" : sttypes,
-          plugins: plugins});
+      if ($elem.data('st-unique') === 'TRUE'){
+        plugins.push('unique');
+      }
+      if ($elem.data('st-sort') === 'TRUE'){
+        plugins.push('sort');
+      }
+      if ($elem.data('st-wholerow') === 'TRUE'){
+        plugins.push('wholerow');
+      }
+      if ($elem.data('st-contextmenu') === 'TRUE'){
+        plugins.push('contextmenu');
+      }
+      var sttypes = null;
+      if(typeof window[el.id + "_sttypes"] !== 'undefined'){
+        sttypes = window[el.id + "_sttypes"];
+      }
+      var tree = $(el).jstree({
+        'core': {
+          "check_callback" : ($elem.data('st-dnd') === 'TRUE') || ($elem.data('st-contextmenu') === 'TRUE'),
+          'multiple': ($elem.data('st-multiple') === 'TRUE'),
+          'animation': $elem.data('st-animation'),
+          'themes': {
+            'name': $elem.data('st-theme'), 
+            'responsive': true, 
+            'stripes': ($elem.data('st-theme-stripes') === 'TRUE'),
+            'icons': ($elem.data('st-theme-icons') === 'TRUE'),
+            'dots': ($elem.data('st-theme-dots') === 'TRUE') 
+          }
+        },
+        "types" : sttypes,
+        plugins: plugins});
     }
   });
   Shiny.outputBindings.register(treeOutput, 'shinyTree.treeOutput');
@@ -44,93 +67,64 @@ var shinyTree = function(){
       return $(scope).find(".shiny-tree");
     },
     getType: function(){
-      return "shinyTree"
+      return "shinyTree";
     },
     getValue: function(el, keys) {
       /**
        * Prune an object recursively to only include the specified keys.
-       * 'li_attr' is a special key that will actually map to 'li_attrs.class' and
-       * will be called 'class' in the output.
+       * Then add any data.
        **/
-      var prune = function(arr, keys){
+        var treeid = $('#' + el.id);
+        var fixOutput = function(arr, keys){
         var arrToObj = function(ar){
           var obj = {};
           $.each(ar, function(i, el){
+            //add the data for this node
+            var data = {};
+            $.each($(treeid).jstree(true).get_node(el.id).data, function(key, val){
+              if (typeof val === 'string'){
+                data[key] = val.trim();
+              } else {
+                data[key] = val; 
+              }
+            });
+            el.data = data;
             obj['' + i] = el;
-          })
+          });
           return obj;
-        }
+        };
         
         var toReturn = [];
-        // Ensure 'children' property is retained.
-        keys.push('children');
-        
         $.each(arr, function(i, obj){
           if (obj.children && obj.children.length > 0){
-            obj.children = arrToObj(prune(obj.children, keys));
+            obj.children = arrToObj(fixOutput(obj.children, keys));
           }
+
           var clean = {};
-          
           $.each(obj, function(key, val){
             if (keys.indexOf(key) >= 0) {
-              //console.log(key + ": " + val)
-              
-              if (key === 'li_attr') { // We don't really want, just the stid and class attr
-                if (val.stid){
-                  //console.log("stid (li_attr): " + val.stid)
-                  if (typeof val.stid === 'string'){
-                    // TODO: We don't really want to trim but have to b/c of Shiny's pretty-printing
-                    clean["stid"] = val.stid.trim();
-                  } else {
-                    clean["stid"] = val.stid; 
-                  }
-                }
-                
-                if (val.class) {
-                  //console.log("stclass (li_attr): " + val.class)
-                  if (typeof val.class === 'string'){
-                    // TODO: We don't really want to trim but have to b/c of Shiny's pretty-printing
-                    clean["stclass"] = val.class.trim();
-                  } else {
-                    clean["stclass"] = val.class; 
-                  }
-                }
-                
-                //if (!val.class){
-                //  console.log(key + ": " + val)
-                //  // Skip without adding element.
-                //  return;
-                //}
-                
-                //if (val.class){
-                //  val = val.class;
-                //  key = 'class';
-                //}
-              } else {
-              
                 if (typeof val === 'string'){
-                  // TODO: We don't really want to trim but have to b/c of Shiny's pretty-printing.
                   clean[key] = val.trim();
                 } else {
                   clean[key] = val; 
                 }
               }
-            }
           });
           
           toReturn.push(clean);
         });
-        return arrToObj(toReturn);
-      }
+        result = arrToObj(toReturn);
+        return arrToObj(result);
+      };
       
       var tree = $.jstree.reference(el);
       if (tree) { // May not be loaded yet.
         if(tree.get_container().find("li").length>0) { // The tree may be initialized but empty
           var js = tree.get_json();
-          var pruned =  prune(js, ['id', 'state', 'text', 'li_attr']);
+          var fixed = fixOutput(js, ['id', 'state', 'text','children']);
           callbackCounter++;
-          pruned.callbackCounter = callbackCounter;
-          return pruned;
+          fixed.callbackCounter = callbackCounter;
+          return fixed;
         }
       }
     },
@@ -151,11 +145,35 @@ var shinyTree = function(){
       $(el).on("ready.jstree", function(e){
         // Initialize the data.
         callback();
-      })
+      });
       
       $(el).on("move_node.jstree", function(e){
         callback();
-      })
+      });
+      
+      // callbacks for contextmenue
+      $(el).on("create_node.jstree", function(e){
+        callback();
+      });
+      $(el).on("rename_node.jstree", function(e){
+        callback();
+      });
+      $(el).on("delete_node.jstree", function(e){
+        callback();
+      });
+      $(el).on("copy_node.jstree", function(e){
+        callback();
+      });
+      $(el).on("cut.jstree", function(e){
+        callback();
+      });
+      $(el).on("copy.jstree", function(e){
+        callback();
+      });
+      $(el).on("paste.jstree", function(e){
+        callback();
+      });
+      
     },
     unsubscribe: function(el) {
       $(el).off(".jstree");
@@ -173,7 +191,7 @@ var shinyTree = function(){
   
   var exports = {};
   
-  exports.initSearch = function(treeId, searchId){
+  exports.initSearch = function(treeId, searchId, searchtime){
     $(function(){
       var to = false;
       $('#' + searchId).keyup(function () {
@@ -181,7 +199,7 @@ var shinyTree = function(){
         to = setTimeout(function () {
           var v = $('#' + searchId).val();
           $.jstree.reference('#' + treeId).search(v);
-        }, 250);
+        }, searchtime);
       });
     });    
   }
